from plans.validators import ModelCountValidator

from .models import Foo


class MaxFoosValidator(ModelCountValidator):
    code = "MAX_FOO_COUNT"
    model = Foo

    def get_queryset(self, user):
        return super(MaxFoosValidator, self).get_queryset(user).filter(user=user)


max_foos_validator = MaxFoosValidator()



import datetime
from decimal import Decimal
from io import StringIO

from django.core.management import call_command
from django.test import TestCase
from model_bakery import baker

from plans.models import AbstractRecurringUserPlan


class ManagementCommandTests(TestCase):
    def test_command_output(self):
        out = StringIO()
        call_command("autorenew_accounts", stdout=out)
        self.assertEqual(out.getvalue(), "Starting renewal\nNo accounts autorenewed\n")

    def test_renewal(self):
        _make_user(
            userplan__expire=datetime.date(2020, 1, 2),
            userplan__recurring__renewal_triggered_by=AbstractRecurringUserPlan.RENEWAL_TRIGGERED_BY.TASK,
            userplan__recurring__token_verified=True,
        )
        out = StringIO()
        call_command("autorenew_accounts", stdout=out)
        self.assertEqual(
            out.getvalue().strip(),
            "Starting renewal\nAccounts submitted to renewal:\n\tinternal-payment-recurring\t\ttestuser",
        )

    def test_renewal_triggered_by_user(self):
        _make_user(
            userplan__expire=datetime.date(2020, 1, 2),
            userplan__recurring__renewal_triggered_by=AbstractRecurringUserPlan.RENEWAL_TRIGGERED_BY.USER,
            userplan__recurring__token_verified=True,
        )
        out = StringIO()
        call_command("autorenew_accounts", stdout=out)
        self.assertEqual(out.getvalue(), "Starting renewal\nNo accounts autorenewed\n")

    def test_renewal_triggered_by_other(self):
        _make_user(
            userplan__expire=datetime.date(2020, 1, 2),
            userplan__recurring__renewal_triggered_by=AbstractRecurringUserPlan.RENEWAL_TRIGGERED_BY.OTHER,
            userplan__recurring__token_verified=True,
        )
        out = StringIO()
        call_command("autorenew_accounts", stdout=out)
        self.assertEqual(out.getvalue(), "Starting renewal\nNo accounts autorenewed\n")

    def test_renewal_providers(self):
        _make_user(
            userplan__expire=datetime.date(2020, 1, 2),
            userplan__recurring__renewal_triggered_by=AbstractRecurringUserPlan.RENEWAL_TRIGGERED_BY.TASK,
            userplan__recurring__payment_provider="internal-payment-recurring",
            userplan__recurring__token_verified=True,
        )
        out = StringIO()
        call_command("autorenew_accounts", providers="foo", stdout=out)
        self.assertEqual(out.getvalue(), "Starting renewal\nNo accounts autorenewed\n")


def _make_user(
    userplan__expire,
    userplan__recurring__renewal_triggered_by,
    userplan__recurring__token_verified,
    userplan__recurring__payment_provider="internal-payment-recurring",
):
    user = baker.make("User", username="testuser")
    plan_pricing = baker.make("PlanPricing", plan=baker.make("Plan", name="Foo plan"))
    baker.make(
        "UserPlan",
        user=user,
        plan=plan_pricing.plan,
        recurring__payment_provider=userplan__recurring__payment_provider,
        recurring__amount=Decimal(123),
        recurring__pricing=plan_pricing.pricing,
        recurring__currency="USD",
        recurring__renewal_triggered_by=userplan__recurring__renewal_triggered_by,
        recurring__token_verified=userplan__recurring__token_verified,
        expire=userplan__expire,
    )



import datetime
import logging

from django.conf import settings
from django.contrib.auth import get_user_model

from .base.models import AbstractRecurringUserPlan
from .signals import account_automatic_renewal

User = get_user_model()
logger = logging.getLogger("plans.tasks")


def get_active_plans():
    return (
        User.objects.select_related("userplan")
        .filter(userplan__active=True)
        .exclude(userplan__expire=None)
    )


def autorenew_account(providers=None):
    logger.info("Started automatic account renewal")
    PLANS_AUTORENEW_BEFORE_DAYS = getattr(settings, "PLANS_AUTORENEW_BEFORE_DAYS", 0)
    PLANS_AUTORENEW_BEFORE_HOURS = getattr(settings, "PLANS_AUTORENEW_BEFORE_HOURS", 0)

    accounts_for_renewal = get_active_plans().filter(
        userplan__recurring__renewal_triggered_by=AbstractRecurringUserPlan.RENEWAL_TRIGGERED_BY.TASK,
        userplan__recurring__token_verified=True,
        userplan__expire__lt=datetime.date.today()
        + datetime.timedelta(
            days=PLANS_AUTORENEW_BEFORE_DAYS, hours=PLANS_AUTORENEW_BEFORE_HOURS
        ),
    )

    if providers:
        accounts_for_renewal = accounts_for_renewal.filter(
            userplan__recurring__payment_provider__in=providers
        )

    logger.info(f"{len(accounts_for_renewal)} accounts to be renewed.")

    for user in accounts_for_renewal.all():
        account_automatic_renewal.send(sender=None, user=user)
    return accounts_for_renewal


def expire_account():
    logger.info("Started account expiration")

    expired_accounts = get_active_plans().filter(
        userplan__expire__lt=datetime.date.today()
    )

    for user in expired_accounts.all():
        user.userplan.expire_account()

    notifications_days_before = getattr(settings, "PLANS_EXPIRATION_REMIND", [])

    if notifications_days_before:
        days = map(
            lambda x: datetime.date.today() + datetime.timedelta(days=x),
            notifications_days_before,
        )
        for user in User.objects.select_related("userplan").filter(
            userplan__active=True, userplan__expire__in=days
        ):
            user.userplan.remind_expire_soon()



from django.contrib.auth.decorators import login_required
from django.urls import path

from .views import FooCreateView, FooDeleteView, FooListView

urlpatterns = [
    path("list/", login_required(FooListView.as_view()), name="foo_list"),
    path("add/", login_required(FooCreateView.as_view()), name="foo_add"),
    path("del/<int:pk>/", login_required(FooDeleteView.as_view()), name="foo_del"),
]



# Generated by Django 3.2.11 on 2022-01-19 17:14

from decimal import Decimal

import django.db.models.deletion
import django_countries.fields
import example.sample_plans.models
from django.conf import settings
from django.db import migrations, models

import plans.base.models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Plan",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "order",
                    models.PositiveIntegerField(
                        db_index=True, editable=False, verbose_name="order"
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="name")),
                (
                    "description",
                    models.TextField(blank=True, verbose_name="description"),
                ),
                (
                    "available",
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text="Is still available for purchase",
                        verbose_name="available",
                    ),
                ),
                (
                    "visible",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Is visible in current offer",
                        verbose_name="visible",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(db_index=True, verbose_name="created"),
                ),
                (
                    "url",
                    models.URLField(
                        blank=True,
                        help_text="Optional link to page with more information (for clickable pricing table headers)",
                    ),
                ),
                (
                    "default",
                    models.BooleanField(
                        blank=True,
                        default=None,
                        help_text='Both "Unknown" and "No" means that the plan is not default',
                        null=True,
                    ),
                ),
                (
                    "customized",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="customized",
                    ),
                ),
            ],
            options={
                "verbose_name": "Plan",
                "verbose_name_plural": "Plans",
                "ordering": ("order",),
                "abstract": False,
            },
            bases=(plans.base.models.BaseMixin, models.Model),
        ),
        migrations.CreateModel(
            name="Pricing",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="name")),
                (
                    "period",
                    models.PositiveIntegerField(
                        blank=True,
                        db_index=True,
                        default=30,
                        null=True,
                        verbose_name="period",
                    ),
                ),
                (
                    "url",
                    models.URLField(
                        blank=True,
                        help_text="Optional link to page with more information (for clickable pricing table headers)",
                    ),
                ),
            ],
            options={
                "verbose_name": "Pricing",
                "verbose_name_plural": "Pricings",
                "ordering": ("period",),
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="Quota",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "order",
                    models.PositiveIntegerField(
                        db_index=True, editable=False, verbose_name="order"
                    ),
                ),
                (
                    "codename",
                    models.CharField(
                        db_index=True,
                        max_length=50,
                        unique=True,
                        verbose_name="codename",
                    ),
                ),
                ("name", models.CharField(max_length=100, verbose_name="name")),
                (
                    "unit",
                    models.CharField(blank=True, max_length=100, verbose_name="unit"),
                ),
                (
                    "description",
                    models.TextField(blank=True, verbose_name="description"),
                ),
                (
                    "is_boolean",
                    models.BooleanField(default=False, verbose_name="is boolean"),
                ),
                (
                    "url",
                    models.CharField(
                        blank=True,
                        help_text="Optional link to page with more information (for clickable pricing table headers)",
                        max_length=200,
                    ),
                ),
            ],
            options={
                "verbose_name": "Quota",
                "verbose_name_plural": "Quotas",
                "ordering": ("order",),
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="TestApp",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(example.sample_plans.models.DetailFieldMixin, models.Model),
        ),
        migrations.CreateModel(
            name="UserPlan",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "expire",
                    models.DateField(
                        blank=True,
                        db_index=True,
                        default=None,
                        null=True,
                        verbose_name="expire",
                    ),
                ),
                (
                    "active",
                    models.BooleanField(
                        db_index=True, default=True, verbose_name="active"
                    ),
                ),
                (
                    "plan",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.plan",
                        verbose_name="plan",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "User plan",
                "verbose_name_plural": "Users plans",
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="RecurringUserPlan",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "token",
                    models.CharField(
                        blank=True,
                        default=None,
                        help_text="Token, that will be used for payment renewal. Depends on used payment provider",
                        max_length=255,
                        null=True,
                        verbose_name="recurring token",
                    ),
                ),
                (
                    "payment_provider",
                    models.CharField(
                        blank=True,
                        default=None,
                        help_text="Provider, that will be used for payment renewal",
                        max_length=255,
                        null=True,
                        verbose_name="payment provider",
                    ),
                ),
                (
                    "amount",
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=7,
                        null=True,
                        verbose_name="amount",
                    ),
                ),
                (
                    "tax",
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=4,
                        null=True,
                        verbose_name="tax",
                    ),
                ),
                ("currency", models.CharField(max_length=3, verbose_name="currency")),
                (
                    "renewal_triggered_by",
                    models.IntegerField(
                        choices=[(1, "other"), (2, "user"), (3, "task")],
                        db_index=True,
                        default=2,
                        help_text="The source of the associated plan's renewal (USER = user-initiated renewal, TASK = autorenew_account-task-initiated renewal, OTHER = renewal is triggered using another mechanism).",
                        verbose_name="renewal triggered by",
                    ),
                ),
                (
                    "_has_automatic_renewal_backup_deprecated",
                    models.BooleanField(
                        db_column="has_automatic_renewal",
                        default=False,
                        help_text="Automatic renewal is enabled for associated plan. If False, the plan renewal can be still initiated by user.",
                        verbose_name="has automatic plan renewal",
                    ),
                ),
                (
                    "token_verified",
                    models.BooleanField(
                        default=False,
                        help_text="The recurring token has been verified by at least one payment to be working.",
                        verbose_name="token has been verified by payment",
                    ),
                ),
                ("card_expire_year", models.IntegerField(blank=True, null=True)),
                ("card_expire_month", models.IntegerField(blank=True, null=True)),
                (
                    "card_masked_number",
                    models.CharField(blank=True, max_length=255, null=True),
                ),
                (
                    "pricing",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        help_text="Recurring pricing",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.pricing",
                    ),
                ),
                (
                    "user_plan",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="recurring",
                        to="sample_plans.userplan",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="PlanQuota",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.IntegerField(blank=True, default=1, null=True)),
                (
                    "plan",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.plan",
                    ),
                ),
                (
                    "quota",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.quota",
                    ),
                ),
            ],
            options={
                "verbose_name": "Plan quota",
                "verbose_name_plural": "Plans quotas",
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="PlanPricing",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "price",
                    models.DecimalField(db_index=True, decimal_places=2, max_digits=7),
                ),
                ("order", models.IntegerField(default=0)),
                (
                    "has_automatic_renewal",
                    models.BooleanField(
                        default=False,
                        help_text="Use automatic renewal if possible?",
                        verbose_name="has automatic renewal",
                    ),
                ),
                (
                    "plan",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.plan",
                    ),
                ),
                (
                    "pricing",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.pricing",
                    ),
                ),
                (
                    "visible",
                    models.BooleanField(
                        default=True,
                        help_text="Is visible in current offer",
                        verbose_name="is visible by default",
                    ),
                ),
            ],
            options={
                "verbose_name": "Plan pricing",
                "verbose_name_plural": "Plans pricings",
                "ordering": ("order", "pricing__period"),
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.AddField(
            model_name="plan",
            name="quotas",
            field=models.ManyToManyField(
                through="sample_plans.PlanQuota", to="sample_plans.Quota"
            ),
        ),
        migrations.CreateModel(
            name="Order",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("flat_name", models.CharField(blank=True, max_length=200, null=True)),
                (
                    "created",
                    models.DateTimeField(db_index=True, verbose_name="created"),
                ),
                (
                    "completed",
                    models.DateTimeField(
                        blank=True, db_index=True, null=True, verbose_name="completed"
                    ),
                ),
                (
                    "plan_extended_from",
                    models.DateField(
                        blank=True,
                        help_text="The plan was extended from this date",
                        null=True,
                        verbose_name="plan extended from",
                    ),
                ),
                (
                    "plan_extended_until",
                    models.DateField(
                        blank=True,
                        help_text="The plan was extended until this date",
                        null=True,
                        verbose_name="plan extended until",
                    ),
                ),
                (
                    "amount",
                    models.DecimalField(
                        db_index=True,
                        decimal_places=2,
                        max_digits=7,
                        verbose_name="amount",
                    ),
                ),
                (
                    "tax",
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=4,
                        null=True,
                        verbose_name="tax",
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        default="EUR", max_length=3, verbose_name="currency"
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[
                            (1, "new"),
                            (2, "completed"),
                            (3, "not valid"),
                            (4, "canceled"),
                            (5, "returned"),
                        ],
                        default=1,
                        verbose_name="status",
                    ),
                ),
                (
                    "plan",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="plan_order",
                        to="sample_plans.plan",
                        verbose_name="plan",
                    ),
                ),
                (
                    "pricing",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.pricing",
                        verbose_name="pricing",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "Order",
                "verbose_name_plural": "Orders",
                "ordering": ("-created",),
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="Invoice",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("number", models.IntegerField(db_index=True)),
                ("full_number", models.CharField(max_length=200)),
                (
                    "type",
                    models.IntegerField(
                        choices=[
                            (1, "Invoice"),
                            (2, "Invoice Duplicate"),
                            (3, "Order confirmation"),
                        ],
                        db_index=True,
                        default=1,
                    ),
                ),
                ("issued", models.DateField(db_index=True)),
                (
                    "issued_duplicate",
                    models.DateField(blank=True, db_index=True, null=True),
                ),
                (
                    "selling_date",
                    models.DateField(blank=True, db_index=True, null=True),
                ),
                ("payment_date", models.DateField(db_index=True)),
                ("unit_price_net", models.DecimalField(decimal_places=2, max_digits=7)),
                ("quantity", models.IntegerField(default=1)),
                ("total_net", models.DecimalField(decimal_places=2, max_digits=7)),
                ("total", models.DecimalField(decimal_places=2, max_digits=7)),
                ("tax_total", models.DecimalField(decimal_places=2, max_digits=7)),
                (
                    "tax",
                    models.DecimalField(
                        blank=True,
                        db_index=True,
                        decimal_places=2,
                        max_digits=4,
                        null=True,
                    ),
                ),
                (
                    "rebate",
                    models.DecimalField(
                        decimal_places=2, default=Decimal("0"), max_digits=4
                    ),
                ),
                ("currency", models.CharField(default="EUR", max_length=3)),
                ("item_description", models.CharField(max_length=200)),
                ("buyer_name", models.CharField(max_length=200, verbose_name="Name")),
                (
                    "buyer_street",
                    models.CharField(max_length=200, verbose_name="Street"),
                ),
                (
                    "buyer_zipcode",
                    models.CharField(max_length=200, verbose_name="Zip code"),
                ),
                ("buyer_city", models.CharField(max_length=200, verbose_name="City")),
                (
                    "buyer_country",
                    django_countries.fields.CountryField(
                        default="PL", max_length=2, verbose_name="Country"
                    ),
                ),
                (
                    "buyer_tax_number",
                    models.CharField(
                        blank=True, max_length=200, verbose_name="TAX/VAT number"
                    ),
                ),
                (
                    "shipping_name",
                    models.CharField(max_length=200, verbose_name="Name"),
                ),
                (
                    "shipping_street",
                    models.CharField(max_length=200, verbose_name="Street"),
                ),
                (
                    "shipping_zipcode",
                    models.CharField(max_length=200, verbose_name="Zip code"),
                ),
                (
                    "shipping_city",
                    models.CharField(max_length=200, verbose_name="City"),
                ),
                (
                    "shipping_country",
                    django_countries.fields.CountryField(
                        default="PL", max_length=2, verbose_name="Country"
                    ),
                ),
                ("require_shipment", models.BooleanField(db_index=True, default=False)),
                ("issuer_name", models.CharField(max_length=200, verbose_name="Name")),
                (
                    "issuer_street",
                    models.CharField(max_length=200, verbose_name="Street"),
                ),
                (
                    "issuer_zipcode",
                    models.CharField(max_length=200, verbose_name="Zip code"),
                ),
                ("issuer_city", models.CharField(max_length=200, verbose_name="City")),
                (
                    "issuer_country",
                    django_countries.fields.CountryField(
                        default="PL", max_length=2, verbose_name="Country"
                    ),
                ),
                (
                    "issuer_tax_number",
                    models.CharField(
                        blank=True, max_length=200, verbose_name="TAX/VAT number"
                    ),
                ),
                (
                    "order",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sample_plans.order",
                        verbose_name="order",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "Invoice",
                "verbose_name_plural": "Invoices",
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="BillingInfo",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "tax_number",
                    models.CharField(
                        blank=True, db_index=True, max_length=200, verbose_name="VAT ID"
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True, max_length=200, verbose_name="name"
                    ),
                ),
                ("street", models.CharField(max_length=200, verbose_name="street")),
                ("zipcode", models.CharField(max_length=200, verbose_name="zip code")),
                ("city", models.CharField(max_length=200, verbose_name="city")),
                (
                    "country",
                    django_countries.fields.CountryField(
                        max_length=2, verbose_name="country"
                    ),
                ),
                (
                    "shipping_name",
                    models.CharField(
                        blank=True,
                        help_text="optional",
                        max_length=200,
                        verbose_name="name (shipping)",
                    ),
                ),
                (
                    "shipping_street",
                    models.CharField(
                        blank=True,
                        help_text="optional",
                        max_length=200,
                        verbose_name="street (shipping)",
                    ),
                ),
                (
                    "shipping_zipcode",
                    models.CharField(
                        blank=True,
                        help_text="optional",
                        max_length=200,
                        verbose_name="zip code (shipping)",
                    ),
                ),
                (
                    "shipping_city",
                    models.CharField(
                        blank=True,
                        help_text="optional",
                        max_length=200,
                        verbose_name="city (shipping)",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "Billing info",
                "verbose_name_plural": "Billing infos",
                "abstract": False,
            },
            bases=(
                example.sample_plans.models.DetailFieldMixin,
                plans.base.models.BaseMixin,
                models.Model,
            ),
        ),
    ]



from fabric.api import local, task


@task
def freeze_fixtures():
    local(
        "python manage.py dumpdata auth.User plans.BillingInfo plans.Invoice "
        "plans.Order plans.Plan plans.Pricing plans.PlanPricing plans.Quota "
        "plans.PlanQuota plans.UserPlan > example/foo/fixtures/initial_data.json",
    )



import logging
from decimal import Decimal
from urllib.error import URLError
from xml.sax import SAXParseException

import stdnum.eu.vat
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.utils.html import format_html
from requests.exceptions import ConnectionError
from suds import WebFault
from suds.transport import TransportError

from plans.taxation import TaxationPolicy
from plans.utils import country_code_transform

logger = logging.getLogger("plans.taxation.eu.vies")


class EUTaxationPolicy(TaxationPolicy):
    """
    This taxation policy should be correct for all EU countries. It uses following rules:
        * if issuer country is not in EU - assert error,
        * for buyer of the same country as issuer - return issuer tax,
        * for company buyer from EU (with VIES) returns VAT n/a reverse charge,
        * for non-company buyer from EU returns VAT from buyer country,
        * for non-EU buyer return VAT n/a.

    This taxation policy was updated at 1 Jan 2015 after new UE VAT regulations. You should also probably
    register in MOSS system.
    """

    # Standard VAT rates according to
    # http://ec.europa.eu/taxation_customs/resources/documents/taxation/vat/how_vat_works/rates/vat_rates_en.pdf
    # Situation at 1 Jan 2017

    EU_COUNTRIES_VAT = {
        "BE": Decimal("21"),  # Belgium
        "BG": Decimal("20"),  # Bulgaria
        "CZ": Decimal("21"),  # Czech Republic
        "DK": Decimal("25"),  # Denmark
        "DE": Decimal("19"),  # Germany
        "EE": Decimal("20"),  # Estonia
        "EL": Decimal("24"),  # Greece
        "ES": Decimal("21"),  # Spain
        "FR": Decimal("20"),  # France
        "HR": Decimal("25"),  # Croatia
        "IE": Decimal("23"),  # Ireland
        "IT": Decimal("22"),  # Italy
        "CY": Decimal("19"),  # Cyprus
        "LV": Decimal("21"),  # Latvia
        "LT": Decimal("21"),  # Lithuania
        "LU": Decimal("17"),  # Luxembourg
        "HU": Decimal("27"),  # Hungary
        "MT": Decimal("18"),  # Malta
        "NL": Decimal("21"),  # Netherlands
        "AT": Decimal("20"),  # Austria
        "PL": Decimal("23"),  # Poland
        "PT": Decimal("23"),  # Portugal
        "RO": Decimal("19"),  # Romania
        "SI": Decimal("22"),  # Slovenia
        "SK": Decimal("20"),  # Slovakia
        "FI": Decimal("24"),  # Finland
        "SE": Decimal("25"),  # Sweden
        # 'GB': Decimal('20'),  # United Kingdom (Great Britain)
    }

    @classmethod
    def is_in_EU(cls, country_code):
        return country_code_transform(country_code).upper() in cls.EU_COUNTRIES_VAT

    @classmethod
    def get_default_tax(cls):
        issuer_country_code = cls.get_issuer_country_code()
        issuer_country_code = country_code_transform(issuer_country_code)
        try:
            return cls.EU_COUNTRIES_VAT[issuer_country_code]
        except KeyError:
            raise ImproperlyConfigured(
                "EUTaxationPolicy requires that issuer country is in EU"
            )

    @classmethod
    def get_tax_rate(cls, tax_id, country_code, request=None):
        """
        returns tax rate and if the request was successful.
        """
        country_code = country_code_transform(country_code)
        issuer_country_code = cls.get_issuer_country_code()
        if not cls.is_in_EU(issuer_country_code):
            raise ImproperlyConfigured(
                "EUTaxationPolicy requires that issuer country is in EU"
            )

        if not tax_id and not country_code:
            # No vat id, no country
            return cls.get_default_tax(), True

        elif not tax_id and country_code:
            # Customer is not a company, we know his country

            if cls.is_in_EU(country_code):
                # Customer (private person) is from a EU
                # Customer pays his VAT rate
                return cls.EU_COUNTRIES_VAT[country_code], True
            else:
                # Customer (private person) not from EU
                # VAT n/a
                return None, True

        else:
            # Customer is company, we now country and vat id

            if country_code.upper() == issuer_country_code.upper():
                # Company is from the same country as issuer
                # Normal tax
                return cls.get_default_tax(), True

            if cls.is_in_EU(country_code):
                # Company is from other EU country
                try:
                    vies_result = bool(stdnum.eu.vat.check_vies(tax_id)["valid"])
                    logger.info("TAX_ID=%s RESULT=%s" % (tax_id, vies_result))
                    if tax_id and vies_result:
                        # Company is registered in VIES
                        # Charge back
                        return None, True
                    else:
                        return cls.EU_COUNTRIES_VAT[country_code], True
                except (
                    WebFault,
                    TransportError,
                    stdnum.exceptions.InvalidComponent,
                    ConnectionError,
                    URLError,
                    SAXParseException,
                ) as e:
                    # If we could not connect to VIES or the VAT ID is incorrect
                    if request:
                        messages.warning(
                            request,
                            format_html(
                                "There was an error during determining validity of your VAT ID.<br/>"
                                "If you think, you have european VAT ID and should not be taxed, "
                                "please try resaving billing info later.<br/>"
                                "<br/>"
                                "European VAT Information Exchange System throw following error: {}",
                                e,
                            ),
                        )
                    logger.exception("TAX_ID=%s" % (tax_id))
                    return cls.EU_COUNTRIES_VAT[country_code], False
            else:
                # Company is not from EU
                # VAT n/a
                return None, True



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-04-19 00:55
from __future__ import unicode_literals

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Foo",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default="A new foo", max_length=100)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.11 on 2022-01-13 19:17

import django_countries.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("plans", "0009_auto_20210303_1134"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="buyer_city",
            field=models.CharField(blank=True, max_length=200, verbose_name="City"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="buyer_country",
            field=django_countries.fields.CountryField(
                blank=True, default="PL", max_length=2, verbose_name="Country"
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="buyer_name",
            field=models.CharField(blank=True, max_length=200, verbose_name="Name"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="buyer_street",
            field=models.CharField(blank=True, max_length=200, verbose_name="Street"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="buyer_zipcode",
            field=models.CharField(blank=True, max_length=200, verbose_name="Zip code"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_city",
            field=models.CharField(blank=True, max_length=200, verbose_name="City"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_country",
            field=django_countries.fields.CountryField(
                blank=True, default="PL", max_length=2, verbose_name="Country"
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_name",
            field=models.CharField(blank=True, max_length=200, verbose_name="Name"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_street",
            field=models.CharField(blank=True, max_length=200, verbose_name="Street"),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="shipping_zipcode",
            field=models.CharField(blank=True, max_length=200, verbose_name="Zip code"),
        ),
        migrations.AlterField(
            model_name="plan",
            name="default",
            field=models.BooleanField(
                db_index=True,
                default=None,
                help_text='Both "Unknown" and "No" means that the plan is not default',
                null=True,
                unique=True,
            ),
        ),
    ]



# Generated by Django 2.0.8 on 2018-09-01 15:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("plans", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="order",
            name="plan_extended_from",
            field=models.DateField(
                blank=True,
                null=True,
                verbose_name="plan extended from",
                help_text="The plan was extended from this date",
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="plan_extended_until",
            field=models.DateField(
                blank=True,
                null=True,
                verbose_name="plan extended until",
                help_text="The plan was extended until this date",
            ),
        ),
    ]



from django.core.management import BaseCommand

from plans import tasks


class Command(BaseCommand):
    help = "Autorenew accounts and with recurring payments"

    def add_arguments(self, parser):
        parser.add_argument(
            "--providers",
            nargs="+",
            dest="providers",
            help="Renew only accounts with this providers",
        )

    def handle(self, *args, **options):  # pragma: no cover
        providers = options.get("providers")
        self.stdout.write("Starting renewal")
        renewed_accounts = tasks.autorenew_account(providers)
        if renewed_accounts:
            self.stdout.write("Accounts submitted to renewal:")
            for a in renewed_accounts:
                self.stdout.write(
                    f"\t{a.userplan.recurring.payment_provider}\t{a.email}\t{a}"
                )
        else:
            self.stdout.write("No accounts autorenewed")



from django.contrib import admin
from django.contrib.auth import views
from django.urls import include, path
from django.views.generic.base import TemplateView

admin.autodiscover()

urlpatterns = [
    path("", TemplateView.as_view(template_name="home.html"), name="home"),
    path("admin/", admin.site.urls),
    path("plan/", include("plans.urls")),
    path("accounts/login/", views.LoginView.as_view(), name="login"),
    path(
        "accounts/logout/",
        views.LogoutView.as_view(),
        {"next_page": "/"},
        name="logout",
    ),
    path("foo/", include("example.foo.urls")),
]



import os
import sys
from decimal import Decimal

from django.conf import settings

sys.path[:0] = [os.path.join(os.getcwd(), "demo")]


def pytest_configure(config):
    settings.configure(
        DATABASE_ENGINE="sqlite3",
        DATABASES={
            "default": {
                "NAME": ":memory:",
                "ENGINE": "django.db.backends.sqlite3",
                "TEST_NAME": ":memory:",
            },
        },
        DATABASE_NAME=":memory:",
        TEST_DATABASE_NAME=":memory:",
        INSTALLED_APPS=[
            "django.contrib.auth",
            "django.contrib.contenttypes",
            "django.contrib.admin",
            "django.contrib.sessions",
            "django.contrib.sites",
            "ordered_model",
            "example.foo",
            "plans",
        ],
        ROOT_URLCONF="example.urls",
        DEBUG=False,
        SITE_ID=1,
        TEMPLATE_DEBUG=True,
        USE_TZ=True,
        ALLOWED_HOSTS=["*"],
        PLANS_INVOICE_ISSUER={
            "issuer_name": "My Company Ltd",
            "issuer_street": "48th Suny street",
            "issuer_zipcode": "111-456",
            "issuer_city": "Django City",
            "issuer_country": "PL",
            "issuer_tax_number": "PL123456789",
        },
        PLANS_TAX=Decimal(23.0),
        PLANS_TAXATION_POLICY="plans.taxation.eu.EUTaxationPolicy",
        PLANS_TAX_COUNTRY="PL",
        PLANS_CURRENCY="PLN",
        PLANS_VALIDATORS={
            "MAX_FOO_COUNT": "example.foo.validators.max_foos_validator",
        },
        EMAIL_BACKEND="django.core.mail.backends.console.EmailBackend",
    )



from django.conf import settings
from django.urls import path

from plans.views import (
    AccountActivationView,
    BillingInfoCreateOrUpdateView,
    BillingInfoDeleteView,
    ChangePlanView,
    CreateOrderPlanChangeView,
    CreateOrderView,
    CurrentPlanView,
    FakePaymentsView,
    InvoiceDetailView,
    OrderListView,
    OrderPaymentReturnView,
    OrderView,
    PricingView,
    RedirectToBilling,
    UpgradePlanView,
)

urlpatterns = [
    path("pricing/", PricingView.as_view(), name="pricing"),
    path("account/", CurrentPlanView.as_view(), name="current_plan"),
    path(
        "account/activation/",
        AccountActivationView.as_view(),
        name="account_activation",
    ),
    path("upgrade/", UpgradePlanView.as_view(), name="upgrade_plan"),
    path(
        "order/extend/new/<int:pk>/",
        CreateOrderView.as_view(),
        name="create_order_plan",
    ),
    path(
        "order/upgrade/new/<int:pk>/",
        CreateOrderPlanChangeView.as_view(),
        name="create_order_plan_change",
    ),
    path("change/<int:pk>/", ChangePlanView.as_view(), name="change_plan"),
    path("order/", OrderListView.as_view(), name="order_list"),
    path("order/<int:pk>/", OrderView.as_view(), name="order"),
    path(
        "order/<int:pk>/payment/success/",
        OrderPaymentReturnView.as_view(status="success"),
        name="order_payment_success",
    ),
    path(
        "order/<int:pk>/payment/failure/",
        OrderPaymentReturnView.as_view(status="failure"),
        name="order_payment_failure",
    ),
    # Redirect for backward compatibility:
    path("billing/create/", RedirectToBilling.as_view(), name="billing_info_create"),
    # Redirect for backward compatibility:
    path("billing/update/", RedirectToBilling.as_view(), name="billing_info_update"),
    path("billing/", BillingInfoCreateOrUpdateView.as_view(), name="billing_info"),
    path(
        "billing/delete/", BillingInfoDeleteView.as_view(), name="billing_info_delete"
    ),
    path(
        "invoice/<int:pk>/preview/html/",
        InvoiceDetailView.as_view(),
        name="invoice_preview_html",
    ),
]

if getattr(settings, "DEBUG", False) or getattr(settings, "ENABLE_FAKE_PAYMENTS", True):
    urlpatterns += [
        path(
            "fakepayments/<int:pk>/", FakePaymentsView.as_view(), name="fake_payments"
        ),
    ]



from fabric.api import lcd, local, task


@task
def push_lang():
    with lcd("demo/example"):
        local("django-admin.py makemessages -l en")

    with lcd("./plans"):
        local("django-admin.py makemessages -l en")

    local("tx push -s")


@task
def pull_lang():
    local("tx pull")



from plans.tests.tests import BillingInfoTestCase as BaseBillingInfoTestCase
from plans.tests.tests import BillingInfoViewTestCase as BaseBillingInfoViewTestCase
from plans.tests.tests import ConcurrentTestInvoice as BaseConcurrentTestInvoice
from plans.tests.tests import CreateOrderViewTestCase as BaseCreateOrderViewTestCase
from plans.tests.tests import EUTaxationPolicyTestCase as BaseEUTaxationPolicyTestCase
from plans.tests.tests import OrderTestCase as BaseOrderTestCase
from plans.tests.tests import PlanChangePolicyTestCase as BasePlanChangePolicyTestCase
from plans.tests.tests import PlansTestCase as BasePlansTestCase
from plans.tests.tests import RecurringPlansTestCase as BaseRecurringPlansTestCase
from plans.tests.tests import (
    StandardPlanChangePolicyTestCase as BaseStandardPlanChangePolicyTestCase,
)
from plans.tests.tests import TasksTestCase as BaseTasksTestCase
from plans.tests.tests import TestInvoice as BaseTestInvoice
from plans.tests.tests import ValidatorsTestCase as BaseValidatorsTestCase


class PlanTestCase(BasePlansTestCase):
    pass


class TestInvoice(BaseTestInvoice):
    pass


class ConcurrentTestInvoice(BaseConcurrentTestInvoice):
    pass


class OrderTestCase(BaseOrderTestCase):
    pass


class PlanChangePolicyTestCase(BasePlanChangePolicyTestCase):
    pass


class StandardPlanChangePolicyTestCase(BaseStandardPlanChangePolicyTestCase):
    pass


class EUTaxationPolicyTestCase(BaseEUTaxationPolicyTestCase):
    pass


class BillingInfoTestCase(BaseBillingInfoTestCase):
    pass


class CreateOrderViewTestCase(BaseCreateOrderViewTestCase):
    pass


class ValidatorsTestCase(BaseValidatorsTestCase):
    pass


class BillingInfoViewTestCase(BaseBillingInfoViewTestCase):
    pass


class RecurringPlansTestCase(BaseRecurringPlansTestCase):
    pass


class TasksTestCase(BaseTasksTestCase):
    pass


del BaseBillingInfoTestCase
del BaseBillingInfoViewTestCase
del BaseConcurrentTestInvoice
del BaseCreateOrderViewTestCase
del BaseEUTaxationPolicyTestCase
del BaseOrderTestCase
del BasePlanChangePolicyTestCase
del BasePlansTestCase
del BaseRecurringPlansTestCase
del BaseStandardPlanChangePolicyTestCase
del BaseTasksTestCase
del BaseTestInvoice
del BaseValidatorsTestCase



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# Django settings for example project.
import os
from decimal import Decimal

EMAIL_FROM = "Test <test@server.com>"

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SITE_ROOT = os.path.dirname(os.path.realpath(__file__))

DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

ALLOWED_HOSTS = ["*"]

MANAGERS = ADMINS

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        "NAME": "postgres",  # Or path to database file if using sqlite3.
        "USER": "postgres",  # Not used with sqlite3.
        "PASSWORD": "postgres",  # Not used with sqlite3.
        "HOST": "localhost",  # Set to empty string for localhost. Not used with sqlite3.
        "PORT": "5432",  # Set to empty string for default. Not used with sqlite3.
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
    }
}

TIME_ZONE = "America/Chicago"
USE_TZ = True
LANGUAGE_CODE = "en"
SITE_ID = 1
USE_I18N = True

LANGUAGES = (("en", "English"),)

MEDIA_ROOT = ""
MEDIA_URL = ""
STATIC_ROOT = ""
STATIC_URL = "/static/"
ADMIN_MEDIA_PREFIX = "/static/admin/"
STATICFILES_DIRS = (os.path.join(SITE_ROOT, "static"),)

STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    #    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

SECRET_KEY = "l#^#iad$8$4=dlh74$!xs=3g4jb(&j+y6*ozy&8k1-&d+vruzy"

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(SITE_ROOT, "templates"),
        ],
        "APP_DIRS": False,
        "OPTIONS": {
            "debug": True,
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.template.context_processors.request",
                "django.contrib.messages.context_processors.messages",
                "plans.context_processors.account_status",
            ],
            "loaders": [
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ],
        },
    },
]


ROOT_URLCONF = "example.urls"
WSGI_APPLICATION = "example.wsgi.application"

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "ordered_model",
    "bootstrap3",
    "django_concurrent_tests",
    "plans",
    "example.foo",
    "django_extensions",
    "sequences.apps.SequencesConfig",
]

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "class": "django.utils.log.AdminEmailHandler",
        },
    },
    "loggers": {
        "django.request": {
            "handlers": ["mail_admins"],
            "level": "ERROR",
            "propagate": True,
        },
    },
}


# This is required for django-plans

PLANS_INVOICE_ISSUER = {
    "issuer_name": "My Company Ltd",
    "issuer_street": "48th Suny street",
    "issuer_zipcode": "111-456",
    "issuer_city": "Django City",
    "issuer_country": "PL",
    "issuer_tax_number": "PL123456789",
}

PLANS_TAX = Decimal("23.0")
PLANS_TAXATION_POLICY = "plans.taxation.eu.EUTaxationPolicy"
PLANS_TAX_COUNTRY = "PL"

PLANS_VALIDATORS = {
    "MAX_FOO_COUNT": "example.foo.validators.max_foos_validator",
}

MANAGE_PY_PATH = os.environ.get("MANAGE_PY_PATH", "./manage.py")

PLANS_CURRENCY = "EUR"

EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

LOGIN_REDIRECT_URL = "/foo/list/"

ENABLE_FAKE_PAYMENTS = True

if os.environ.get("SAMPLE_APP", False):
    INSTALLED_APPS.remove("plans")
    INSTALLED_APPS.append("example.sample_plans")

    PLANS_PLAN_MODEL = "sample_plans.Plan"
    PLANS_BILLINGINFO_MODEL = "sample_plans.BillingInfo"
    PLANS_USERPLAN_MODEL = "sample_plans.UserPlan"
    PLANS_PRICING_MODEL = "sample_plans.Pricing"
    PLANS_PLANPRICING_MODEL = "sample_plans.PlanPricing"
    PLANS_QUOTA_MODEL = "sample_plans.Quota"
    PLANS_PLANQUOTA_MODEL = "sample_plans.PlanQuota"
    PLANS_ORDER_MODEL = "sample_plans.Order"
    PLANS_INVOICE_MODEL = "sample_plans.Invoice"
    PLANS_RECURRINGUSERPLAN_MODEL = "sample_plans.RecurringUserPlan"

    # Celery auto detects tasks only from INSTALLED_APPS
    CELERY_IMPORTS = ("openwisp_notifications.tasks",)



def import_name(name):
    """import module given by str or pass the module if it is not str"""
    if isinstance(name, str):
        components = name.split(".")
        mod = __import__(
            ".".join(components[0:-1]), globals(), locals(), [components[-1]]
        )
        return getattr(mod, components[-1])
    else:
        return name



# Generated by Django 4.0.6 on 2022-07-28 13:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("plans", "0011_auto_20220208_1344"),
    ]

    operations = [
        migrations.AddField(
            model_name="planpricing",
            name="visible",
            field=models.BooleanField(
                default=True,
                help_text="Is visible in current offer",
                verbose_name="is visible by default",
            ),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

from plans import __version__ as VERSION

with open("README.rst") as file:
    long_description = file.read()

setup(
    name="django-plans",
    version=VERSION,
    description="Pluggable django app for managing pricing plans with quotas and accounts expiration",
    long_description=long_description,
    author="Krzysztof Dorosz",
    author_email="cypreess@gmail.com",
    url="https://github.com/cypreess/django-plans",
    license="MIT",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
    ],
    install_requires=[
        "django-countries>=4.5",
        "pytz>=2017.2",
        "django-ordered-model>=1.4.1",
        "python-stdnum",
        "django-next-url-mixin>=0.1.0",
        "suds",
        "django-sequences",
        "swapper~=1.3.0",
        "six",
    ],
    extras_require={
        "i18n": [
            "django-modeltranslation>=0.5b1",
        ],
    },
    include_package_data=True,
    zip_safe=False,
)



from django.db import models

from plans.base.models import (
    AbstractBillingInfo,
    AbstractInvoice,
    AbstractOrder,
    AbstractPlan,
    AbstractPlanPricing,
    AbstractPlanQuota,
    AbstractPricing,
    AbstractQuota,
    AbstractRecurringUserPlan,
    AbstractUserPlan,
)


class DetailFieldMixin:
    # Tests additional field can be added to the models
    detail = models.CharField(max_length=50, null=True, blank=True)


class BillingInfo(DetailFieldMixin, AbstractBillingInfo):
    class Meta(AbstractBillingInfo.Meta):
        abstract = False


class Invoice(DetailFieldMixin, AbstractInvoice):
    class Meta(AbstractInvoice.Meta):
        abstract = False


class Order(DetailFieldMixin, AbstractOrder):
    class Meta(AbstractOrder.Meta):
        abstract = False


class Plan(AbstractPlan):
    # Test existing fields can be modified
    default = models.BooleanField(
        help_text=AbstractPlan._meta.get_field("default").help_text,
        default=AbstractPlan._meta.get_field("default").default,
        null=True,
        blank=True,
    )

    class Meta(AbstractPlan.Meta):
        abstract = False


class PlanQuota(DetailFieldMixin, AbstractPlanQuota):
    class Meta(AbstractPlanQuota.Meta):
        abstract = False


class PlanPricing(DetailFieldMixin, AbstractPlanPricing):
    class Meta(AbstractPlanPricing.Meta):
        abstract = False


class Pricing(DetailFieldMixin, AbstractPricing):
    class Meta(AbstractPricing.Meta):
        abstract = False


class Quota(DetailFieldMixin, AbstractQuota):
    class Meta(AbstractQuota.Meta):
        abstract = False


class RecurringUserPlan(DetailFieldMixin, AbstractRecurringUserPlan):
    class Meta(AbstractRecurringUserPlan.Meta):
        abstract = False


class UserPlan(DetailFieldMixin, AbstractUserPlan):
    class Meta(AbstractUserPlan.Meta):
        abstract = False


class TestApp(DetailFieldMixin, models.Model):
    class Meta:
        abstract = False



from django.forms import HiddenInput, ModelForm

from .models import Foo
from .validators import max_foos_validator


class FooForm(ModelForm):
    class Meta:
        model = Foo
        fields = "__all__"
        widgets = {
            "user": HiddenInput,
        }

    def clean(self):
        cleaned_data = super(FooForm, self).clean()
        max_foos_validator(cleaned_data["user"], add=1)
        return cleaned_data



# Generated by Django 3.0.6 on 2020-06-09 14:05

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("plans", "0006_auto_20200504_1541"),
    ]

    operations = [
        migrations.AddField(
            model_name="recurringuserplan",
            name="card_masked_number",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
